/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xmlmap;

import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Array;
import java.net.URL;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactoryConfigurationError;

import org.ow2.opensuit.xmlmap.impl.IXmlModelMessage;
import org.ow2.opensuit.xmlmap.impl.InitializableObjInfoHolder;
import org.ow2.opensuit.xmlmap.impl.ObjInfo;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;
import org.ow2.opensuit.xmlmap.mapping.ElementMapping;
import org.ow2.opensuit.xmlmap.mapping.FieldMapping;
import org.ow2.opensuit.xmlmap.schema.ISchemaElement;
import org.ow2.opensuit.xmlmap.schema.SchemasManager;
import org.ow2.opensuit.xmlmap.utils.XsdHelper;
import org.xml.sax.SAXException;



/**
 * The base class for instantiating an object model that conform to XML mapping rules.
 */
public class XmlEditor extends InitializableObjInfoHolder
{
	private IInstantiationContext validSupport;
	private SchemasManager manager;
	private Object root;
	
	public XmlEditor(IInstantiationContext iSupport)
    {
		validSupport = iSupport;
    }
	/*
	// ===============================================================================
	// === Internal info <--> obj mapping methods
	// ===============================================================================
	public ObjInfo getInfoFromObj(Object iObj)
	{
		for(int i=0; i<_infos.size(); i++)
		{
			if(iObj == ((ObjInfo)_infos.elementAt(i)).getObject())
				return (ObjInfo)_infos.elementAt(i);
		}
		return null;
	}
	public ObjInfo getInfoFromId(String iPathId)
	{
		for(int i=0; i<_infos.size(); i++)
		{
			if(((ObjInfo)_infos.elementAt(i)).getIdPath().equals(iPathId))
				return (ObjInfo)_infos.elementAt(i);
		}
		return null;
	}
	// ===============================================================================
	// === Messages management methods
	// ===============================================================================
	private int countMessages()
	{
		int nbErrs = 0;
		for(int i=0; i<_infos.size(); i++)
		{
			ObjInfo oi = (ObjInfo)_infos.elementAt(i);
			nbErrs += oi.countMessages();
		}
		return nbErrs;
	}

	public void dumpMessages(PrintWriter iPrinter)
	{
		for(int i=0; i<_infos.size(); i++)
			((ObjInfo)_infos.elementAt(i)).dumpMessages(iPrinter);
		iPrinter.flush();
	}
	public boolean hasMessages()
	{
		return countMessages() > 0;
	}
	public IXmlModelMessage[] getAllMessages()
	{
		Vector v = new Vector();
		for(int i=0; i<_infos.size(); i++)
		{
			ObjInfo oi = (ObjInfo)_infos.elementAt(i);
			if(oi.countMessages() > 0)
			{
				Enumeration enu = oi.getMessages();
				while(enu.hasMoreElements())
					v.addElement(enu.nextElement());
			}
		}
		IXmlModelMessage[] ret = new IXmlModelMessage[v.size()];
		v.copyInto(ret);
		return ret;
	}
	public IXmlModelMessage[] getAllElementMessages(Object iXmlElement)
	{
		ObjInfo oi = getInfoFromObj(iXmlElement);
		if(oi == null)
			return null;
		if(oi.countMessages() == 0)
			return null;
		Vector v = new Vector();
		Enumeration enu = oi.getMessages();
		while(enu.hasMoreElements())
			v.addElement(enu.nextElement());
		
		IXmlModelMessage[] ret = new IXmlModelMessage[v.size()];
		v.copyInto(ret);
		return ret;
	}
	public IXmlModelMessage[] getGlobalElementMessages(Object iXmlElement)
	{
		ObjInfo oi = getInfoFromObj(iXmlElement);
		if(oi == null || oi.countMessages() == 0)
			return null;
		
		Vector v = new Vector();
		Enumeration enu = oi.getMessages();
		while(enu.hasMoreElements())
		{
			IXmlModelMessage msg = (IXmlModelMessage)enu.nextElement();
			if(msg.getMappingName() == null)
			{
				v.addElement(msg);
			}
		}
		if(v == null)
			return null;
		
		IXmlModelMessage[] ret = new IXmlModelMessage[v.size()];
		v.copyInto(ret);
		return ret;
	}
	public IXmlModelMessage[] getMappingElementMessages(Object iXmlElement, String iMappingName)
	{
		ObjInfo oi = getInfoFromObj(iXmlElement);
		if(oi == null || oi.countMessages() == 0)
			return null;
		
		Vector v = new Vector();
		Enumeration enu = oi.getMessages();
		while(enu.hasMoreElements())
		{
			IXmlModelMessage msg = (IXmlModelMessage)enu.nextElement();
			if(iMappingName.equals(msg.getMappingName()))
			{
				v.addElement(msg);
			}
		}
		if(v == null)
			return null;
		
		IXmlModelMessage[] ret = new IXmlModelMessage[v.size()];
		v.copyInto(ret);
		return ret;
	}
	*/
	// ===============================================================================
	// === Model Lifecycle methods (load, save, ...)
	// ===============================================================================
	public void load(URL xmlUrl) throws ParserConfigurationException, SAXException, IOException
	{
		// TODO: what to do with XML errors?
		XmlInstantiator model = XmlInstantiator.instantiate(xmlUrl, validSupport);
		manager = model.getSchemas();
		root = model.getRoot();
		revalidate();
	}
	public void save(OutputStream xmlOutput) throws UnsupportedEncodingException, ParserConfigurationException, TransformerFactoryConfigurationError, TransformerException
	{
		XmlSerializer.serialize(manager, root, xmlOutput);
	}
	public Object getRoot()
	{
		return root;
	}
	public SchemasManager getSchemas()
	{
		return manager;
	}
	protected Object instantiateFile(URL xmlFile, ObjInfo parentInfo)
			throws Exception {
		// TODO Auto-generated method stub
		return null;
	}
	
	// ===============================================================================
	// === Model manipulation methods
	// ===============================================================================
	public void delete(Object iObj)
	{
		//TODO
	}
	public void changeType(Object iObj, Class<?> iNewType)
	{
		//TODO
	}
	public void changeValues(Object iObj, FieldMapping[] iFields, Object[] iValues)
	{
		
	}
	public void add(Object iParent, FieldMapping iMapping, Object iChild)
	{
		//TODO
	}
	// TODO: cut, copy, paste
	
	// ===============================================================================
	// === Model validation and glue
	// ===============================================================================
	private void revalidate()
	{
		// --- clean all
		removeAllInfo();
		// --- glue and check
		ObjInfo rootInfo = new ObjInfo(null, root);
		addInfo(rootInfo);
		glueAndCheckObj(rootInfo);
		
		// --- have model validate itself
		initializeAll(validSupport);
	}

	private void glueAndCheckObj(ObjInfo info)
	{
		Object obj = info.getObject();
		ObjInfo parent = info.getParent();
		
		ISchemaElement schemaElt = manager.getElement(obj.getClass());
		if(schemaElt == null)
		{
			info.addMessage(null, IXmlModelMessage.INST_ERROR, "Object not found in any declared schema.");
			return;
		}
		
		// --- extract XmlMappings
		ElementMapping mappings = (ElementMapping)schemaElt.getSchema().getMethod().getElementMappings(obj.getClass(), true);
		
		// --- check and glue (root, parent and ancestor mappings)
		if(parent != null)
		{
//			// =========================================================
//			// === Process Root Mappings
//			// =========================================================
//			FieldMapping[] rootMappings = mappings.getMappings(FieldMapping.ROOT);
//			for(int i=0; i<rootMappings.length; i++)
//			{
//				FieldMapping fieldMapping = rootMappings[i];
//				if(!fieldMapping.getField().getType().isInstance(root))
//				{
//					info.addMessage(fieldMapping.getName(), IXmlModelMessage.INST_ERROR, "Root field '"+fieldMapping.getField().getName()+"' type ("+fieldMapping.getBaseClass().getName()+") is not compatible with the root type: "+root.getClass().getName());
//				}
//				else
//				{
//					try
//					{
//						fieldMapping.getField().setAccessible(true);
//						fieldMapping.getField().set(obj, root);
//					}
//					catch(IllegalArgumentException e)
//					{
//						info.addMessage(fieldMapping.getName(), IXmlModelMessage.INST_ERROR, "Root field '"+fieldMapping.getField().getName()+"' type ("+fieldMapping.getBaseClass().getName()+") is not compatible with the root type: "+root.getClass().getName());
//					}
//					catch(IllegalAccessException e)
//					{
//						info.addMessage(fieldMapping.getName(), IXmlModelMessage.INST_ERROR, "Root field '"+fieldMapping.getField().getName()+"' cannot be set in class '"+obj.getClass().getName()+"'.");
//					}
//				}
//			}
//			
//			// =========================================================
//			// === Process Parent Mappings
//			// =========================================================
////			Object parent = iAncestors.lastElement();
//			FieldMapping[] parentMappings = mappings.getMappings(FieldMapping.PARENT);
//			for(int i=0; i<parentMappings.length; i++)
//			{
//				FieldMapping fieldMapping = (FieldMapping)parentMappings[i];
//				if(!fieldMapping.getField().getType().isInstance(parent.getObject()))
//				{
//					info.addMessage(fieldMapping.getName(), IXmlModelMessage.INST_WARNING, "Parent field '"+fieldMapping.getField().getName()+"' type ("+fieldMapping.getBaseClass().getName()+") is not compatible with the parent type: "+parent.getObject().getClass().getName()+". Will be null.");
//				}
//				else
//				{
//					try
//					{
//						fieldMapping.getField().setAccessible(true);
//						fieldMapping.getField().set(obj, parent.getObject());
//					}
//					catch(IllegalArgumentException e)
//					{
//						info.addMessage(fieldMapping.getName(), IXmlModelMessage.INST_WARNING, "Parent field '"+fieldMapping.getField().getName()+"' type ("+fieldMapping.getBaseClass().getName()+") is not compatible with the parent type: "+parent.getObject().getClass().getName()+". Will be null.");
//					}
//					catch(IllegalAccessException e)
//					{
//						info.addMessage(fieldMapping.getName(), IXmlModelMessage.INST_ERROR, "Parent field '"+fieldMapping.getField().getName()+"' cannot be set in class '"+obj.getClass().getName()+"'.");
//					}
//				}
//			}
			
			// =========================================================
			// === Process Ancestor Mappings
			// =========================================================
			FieldMapping[] ancestorMappings = mappings.getMappings(FieldMapping.ANCESTOR);
			for(int i=0; i<ancestorMappings.length; i++)
			{
				FieldMapping fieldMapping = (FieldMapping)ancestorMappings[i];
				// --- find first ancestor that matches the type
				ObjInfo ancestor = info.getParent();
				while(ancestor != null && !fieldMapping.getField().getType().isInstance(ancestor.getObject()))
					ancestor = ancestor.getParent();
				
				if(ancestor != null)
				{
					try
					{
						// --- set ancestor
						fieldMapping.getField().setAccessible(true);
						fieldMapping.getField().set(obj, ancestor.getObject());
					}
					catch(IllegalArgumentException e)
					{
						info.addMessage(fieldMapping.getName(), IXmlModelMessage.INST_WARNING, "Ancestor field '"+fieldMapping.getField().getName()+"' type ("+fieldMapping.getBaseClass().getName()+") is not compatible with the parent type: "+ancestor.getObject().getClass().getName()+". Will be null.");
					}
					catch(IllegalAccessException e)
					{
						info.addMessage(fieldMapping.getName(), IXmlModelMessage.INST_ERROR, "Ancestor field '"+fieldMapping.getField().getName()+"' cannot be set in class '"+obj.getClass().getName()+"'.");
					}
				}
				else
				{
					// --- ancestor not found
					info.addMessage(fieldMapping.getName(), IXmlModelMessage.INST_INFO, "No matching ancestor for field '"+fieldMapping.getField().getName()+"' type ("+fieldMapping.getBaseClass().getName()+"). Will be null.");
				}
			}
		}

		// =========================================================
		// === Process Content Mappings
		// =========================================================
		FieldMapping[] contentMappings = mappings.getMappings(FieldMapping.CONTENT);
		if(contentMappings.length > 0)
		{
			for(int i=0; i<contentMappings.length; i++)
			{
				FieldMapping fieldMapping = (FieldMapping)contentMappings[i];
				
				fieldMapping.getField().setAccessible(true);
                try
                {
	                String content = (String)fieldMapping.getField().get(obj);
                }
                catch(Exception e)
                {
                	info.addMessage(fieldMapping.getName(), IXmlModelMessage.INST_ERROR, "Content field '"+fieldMapping.getField().getName()+"' could not be retreived.", e);
					continue;
                }
			}
		}
		// =========================================================
		// === Process Attributes
		// =========================================================
		FieldMapping[] attrMappings = mappings.getMappings(FieldMapping.ATTRIBUTE);
		for(int i=0; i<attrMappings.length; i++)
		{
			FieldMapping fieldMapping = (FieldMapping)attrMappings[i];

			// --- get the value
			fieldMapping.getField().setAccessible(true);
			Object val = null;
            try
            {
	            val = fieldMapping.getField().get(obj);
            }
            catch(Exception e)
            {
            	info.addMessage(fieldMapping.getName(), IXmlModelMessage.INST_ERROR, "Attribute field '"+fieldMapping.getField().getName()+"' could not be retreived.", e);
				continue;
            }
			
			if(val == null)
			{
				if(fieldMapping.isUseRequired()){
					info.addMessage(fieldMapping.getName(), IXmlModelMessage.INST_ERROR, "Attribute '"+fieldMapping.getName()+"' required.");
				}
			}
			else
			{
				// --- try to set the field value
				// --- 1: cast to the expected type
				if(XsdHelper.getSimpleType(fieldMapping.getBaseClass()) != null)
				{
				    
				}
				else if(schemaElt.getSchema().getMethod().isEnumeration(fieldMapping.getBaseClass()))
				{
				    
				}
				else
				{
					// --- unsupported type
					info.addMessage(fieldMapping.getName(), IXmlModelMessage.INST_ERROR, "Attribute '"+fieldMapping.getName()+"' has an unsupported type: "+fieldMapping.getBaseClass()+".");
				}
			}
		}
		
		// =========================================================
		// === Process Children Elements
		// =========================================================
		// --- 2: set fields mapped to single nodes
		FieldMapping[] childMappings = mappings.getMappings(FieldMapping.CHILD_ELT);
		for(int i=0; i<childMappings.length; i++)
		{
			FieldMapping fieldMapping = (FieldMapping)childMappings[i];
			// --- get the value
			fieldMapping.getField().setAccessible(true);
			Object val = null;
            try
            {
	            val = fieldMapping.getField().get(obj);
            }
            catch(Exception e)
            {
				info.addMessage(fieldMapping.getName(), IXmlModelMessage.INST_ERROR, "Child node field '"+fieldMapping.getField().getName()+"' could not be retreived.", e);
				continue;
            }
			
			if(val == null)
			{
				if(fieldMapping.isUseRequired()){
					info.addMessage(fieldMapping.getName(), IXmlModelMessage.INST_ERROR, "Child node '"+fieldMapping.getName()+"' required.");
				}
			}
			else
			{
				if(fieldMapping.getField().getType().isArray())
				{
					// --- array type
					int len = Array.getLength(val);
					if(len < fieldMapping.getMinOccurs())
					{
						info.addMessage(fieldMapping.getName(), IXmlModelMessage.INST_ERROR, "Child node '"+fieldMapping.getName()+"' expects at least "+fieldMapping.getMinOccurs()+" children.");
					}
					else if(len > fieldMapping.getMaxOccurs())
					{
						info.addMessage(fieldMapping.getName(), IXmlModelMessage.INST_ERROR, "Child node '"+fieldMapping.getName()+"' expects at most "+fieldMapping.getMaxOccurs()+" children.");
					}
					
					for(int j=0; j<len; j++)
					{
						Object v = (Object)Array.get(val, j);
						ObjInfo childInfo = new ObjInfo(info, v);
						addInfo(childInfo);
						glueAndCheckObj(childInfo);
					}
				}
				else
				{
					// --- simple type
					ObjInfo childInfo = new ObjInfo(info, (Object)val);
					addInfo(childInfo);
					glueAndCheckObj(childInfo);
				}
			}
		}
		
		// =========================================================
		// === Process Collected Elements
		// =========================================================
		FieldMapping[] collectedMappings = mappings.getMappings(FieldMapping.DIRECT_CHILD_ELT);
		for(int i=0; i<collectedMappings.length; i++)
		{
			FieldMapping fieldMapping = (FieldMapping)collectedMappings[i];
			// --- get the value
			fieldMapping.getField().setAccessible(true);
			Object val = null;
            try
            {
	            val = fieldMapping.getField().get(obj);
            }
            catch(Exception e)
            {
				info.addMessage(fieldMapping.getName(), IXmlModelMessage.INST_ERROR, "Collected node '"+fieldMapping.getField().getName()+"' could not be retreived.", e);
				continue;
            }
			
			if(val == null)
			{
				if(fieldMapping.isUseRequired()){
					info.addMessage(fieldMapping.getName(), IXmlModelMessage.INST_ERROR, "Collected node '"+fieldMapping.getName()+"' required.");
				}
			}
			else
			{
				if(fieldMapping.getField().getType().isArray())
				{
					// --- array type
					int len = Array.getLength(val);
					if(len < fieldMapping.getMinOccurs())
					{
						info.addMessage(fieldMapping.getName(), IXmlModelMessage.INST_ERROR, "Collected node '"+fieldMapping.getName()+"' expects at least "+fieldMapping.getMinOccurs()+" children.");
					}
					else if(len > fieldMapping.getMaxOccurs())
					{
						info.addMessage(fieldMapping.getName(), IXmlModelMessage.INST_ERROR, "Collected node '"+fieldMapping.getName()+"' expects at most "+fieldMapping.getMaxOccurs()+" children.");
					}
					
					for(int j=0; j<len; j++)
					{
						Object v = (Object)Array.get(val, j);
						ObjInfo childInfo = new ObjInfo(info, v);
						addInfo(childInfo);
						glueAndCheckObj(childInfo);
					}
				}
				else
				{
					// --- simple type
					ObjInfo childInfo = new ObjInfo(info, (Object)val);
					addInfo(childInfo);
					glueAndCheckObj(childInfo);
				}
			}
		}
	}
}
