/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xmlmap;

import java.io.FileOutputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.ow2.opensuit.xmlmap.schema.ISchema;
import org.ow2.opensuit.xmlmap.schema.SchemasManager;
import org.ow2.opensuit.xmlmap.utils.XmlHelper;
import org.w3c.dom.Document;



/**
 * This class is a program (main) that allows generating the XML schema (XSD)
 * from Java code that support the implicit mapping (see XmlObject.java) 
 */
public class XsdGenerator
{
	/**
	 * The XsdGenerator program.
	 * <p>Usage:</p>
	 * <pre>XsdGenerator <font color=red>&lt;output_xsd_file&gt;</font> <font color=red>&lt;root_model_package&gt;</font> [ <font color=red>&lt;import_xsd_file&gt;</font> <font color=red>&lt;import_root_package&gt;</font> ]*
	 * <p>Exemple:</p>
	 * <pre>XsdGenerator <font color=red>web/generated.xsd com.francetelecom.malignetv.portail.presentation</font>
	 * 
	 * @param args The program arguments.
	 */
	public static void main(String[] args)
	{
		// --- analyse arguments
		if(args.length < 2)
		{
//			System.out.println("usage: GenXSD [sourcedir] [output_file] [java_package] ( [import_file] [import_package] )*");
			System.out.println("usage: GenXSD [output_xsd_file] [java_package] ( [import_xsd_file] [import_package] )*");
			return;
		}
//		String sourceDir = args[0];
		String xsdFile = args[0];
		String javaPackage = args[1];
		try
		{
			System.out.println("Generate XSD schema from code");
			System.out.println("-----------------------------");
//			System.out.println("  - source code directory : "+sourceDir);
			System.out.println("  - output file           : "+xsdFile);
			System.out.println("  - root code package     : "+javaPackage);

			// --- create schemas
			SchemasManager schemas = new SchemasManager();
			ISchema mainSchema = schemas.declareSchema(getPrefix(xsdFile), xsdFile, javaPackage);
			for(int i=2; i<args.length; i+=2)
			{
				String importXsd = args[i];
				// --- import prefix is the xsd filename (no path, no extension)
				String importPrefix = getPrefix(importXsd);
				String importPack = args[i+1];
				System.out.println("  --> import file : "+importXsd+", prefix : "+importPrefix+", Java package: "+importPack);
				schemas.declareSchema(importPrefix, importXsd, importPack);
			}
			
			System.out.println("");
			
			// --- build main schema from source code
//			mainSchema.buildFromSource(sourceDir);
			mainSchema.buildFromClassPath();
			
			// --- generate XSD DOM
			DocumentBuilder docBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
			Document xmlDoc = docBuilder.newDocument();
			mainSchema.generateXSD(xmlDoc);
//			xsdGen.generateXSD(xmlDoc, 0);
			
			// --- output
			XmlHelper.serializeXml(xmlDoc, new FileOutputStream(xsdFile));
			
			System.out.println("... Done.");
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
	private static String getPrefix(String iXsdFile)
	{
		String prefix = iXsdFile;
		int idx = prefix.lastIndexOf('/');
		
		if(idx >= 0){
			prefix = prefix.substring(idx+1);
		}
		
		idx = prefix.indexOf('.');
		
		if(idx >= 0){
			prefix = prefix.substring(0, idx);
		}
		
		return prefix;
	}
}
