package org.ow2.opensuit.xmlmap.impl;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;

public abstract class InitializableObjInfoHolder extends ObjInfoHolder
{
	// ========================================================================
	protected void initializeAll(IInstantiationContext iValidSupport)
	{
		InitializerImpl initializer = new InitializerImpl(this, iValidSupport);
		initializer.initializeAll();
	}
	protected abstract Object instantiateFile(URL xmlFile, ObjInfo parentInfo) throws Exception;
	
	private class InitializerImpl
	{
		private IInstantiationContext validSupport;
		
		private List<ObjInfo> toInitialize = new ArrayList<ObjInfo>();
		private List<ObjInfo> initializing = new ArrayList<ObjInfo>();
		
		public InitializerImpl(ObjInfoHolder objInfoHolder, IInstantiationContext iValidSupport)
		{
		    this.validSupport = iValidSupport;
			addObjsToInitialize(0);
		}
		private void addObjsToInitialize(int startIndex)
		{
			List<ObjInfo> objInfos = getAllObjInfo();
			for(int i=startIndex; i<objInfos.size(); i++)
			{
				ObjInfo info = (ObjInfo)objInfos.get(i);
				
				if(info.getObject() == null){
					continue;
				}
				
				if(!(info.getObject() instanceof IInitializable)){
					continue;
				}
				
				this.toInitialize.add(info);
			}
		}
		public void initializeAll()
		{
			while(!this.toInitialize.isEmpty())
			{
				ObjInfo info = (ObjInfo)this.toInitialize.get(0);
				// do not check state (must be TO_INTIALIZE)
				doInitialize(info);
			}
		}
		private boolean doCouldInitialize(Object object)
		{
			if(object == null){
				// --- object not instantiated by the XML instantiator
				return false;
			}
			
			ObjInfo info = getInfoFromObj(object);
			
			if(info == null){
				// this was not an instantiated object: exception?
				return true;
			}
			
			if(object instanceof IInitializable)
			{
				// --- is it initialized, initializing or to initialize?
				if(this.initializing.contains(info))
				{
					// --- problem: cyclic dependency
					StringBuffer sb = new StringBuffer("Cyclic dependency found: ");
					for(int i=0; i<this.initializing.size(); i++)
					{
						ObjInfo inf = (ObjInfo)this.initializing.get(i);
						sb.append(inf.getObject().getClass().getName());
						sb.append(" --> ");
					}
					sb.append(info.getObject().getClass().getName());
					
					throw new RuntimeException(sb.toString());
				}
				else if(this.toInitialize.contains(info))
				{
					doInitialize(info);
				}
				//else already initialized
			}
			
			return !info.hasMessages(IInitializationSupport.ERROR);
		}
		private void doInitialize(ObjInfo info)
		{
	//System.out.println("init: "+info.getNicePath());
		    this.toInitialize.remove(info);
			this.initializing.add(info);
			try
			{
				((IInitializable)info.getObject()).initialize(new InitializationSupportImpl(info), this.validSupport);
			}
			catch(Throwable t)
			{
				info.addMessage(null, InitializationSupportImpl.ERROR, "Initialization error.", t);
			}
			this.initializing.remove(info);
		}
		// =======================================================================
		// === IInitializationSupport implementation
		// =======================================================================
		private class InitializationSupportImpl implements IInitializationSupport
		{
			private ObjInfo info;
			
			public InitializationSupportImpl(ObjInfo info)
			{
				this.info = info;
			}
			public void addValidationMessage(Object fromNode, String mappingName, int severity, String message)
			{
				addValidationMessage(fromNode, mappingName, severity, message, null);
			}
			public void addValidationMessage(Object fromNode, String mappingName, int severity, String message, Throwable error)
			{
				if(fromNode == this.info.getObject())
				{
				    this.info.addMessage(mappingName, severity, message, error);
				}
				else
				{
					ObjInfo info2 = getInfoFromObj(fromNode);
					
					if(info2 != null){
						info2.addMessage(mappingName, severity, message, error);
					}
				}
			}
			public boolean initialize(Object object)
			{
				// TODO: check whether the object is an ancestor (forbidden!)
				boolean dependencyCouldInit = doCouldInitialize(object);
	//			if(!dependencyCouldInit)
	//				info.addMessage(null, IInitializationSupport.ERROR, "Dependency failed to initialize: "+object);
				return dependencyCouldInit;
			}
			public Object instantiate(URL xmlFileUrl) throws Exception
			{
				int nbObjsBefore = getAllObjInfo().size();
				try
				{
					return instantiateFile(xmlFileUrl, this.info);
				}
				finally
				{
					// --- update toInitialize list with all created objects
					addObjsToInitialize(nbObjsBefore);
				}
			}
		}
	}
}
