/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xmlmap.impl;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.URL;


public class ModelMessageImpl implements IXmlModelMessage
{
	private ObjInfo objInfo;
	private String mapping;
	private int type;
	private String msg;
	private Throwable error;
	
	public ModelMessageImpl(ObjInfo iObjInfo, String iMappingName, int iType, String iMsg, Throwable error)
	{
		objInfo = iObjInfo;
		mapping  = iMappingName;
		type = iType;
		msg = iMsg;
		this.error = error;
	}
	public Object getObject()
	{
	    return objInfo.getObject();
	}
	public ObjInfo getInfo()
	{
		return objInfo;
	}
	public String getMappingName()
	{
	    return mapping;
	}
	public String getMessage()
	{
		if(error == null){
			return msg;
		}
		
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		pw.println(msg);
		error.printStackTrace(pw);
		
		return sw.toString();
	}
	public int getSeverity()
	{
	    return type;
	}
	public URL getFile()
	{
		return objInfo.getXmlFile();
	}
	public int getLine()
	{
//		return objInfo.getLineNumber();
		return objInfo.getMappingLocation(mapping);
	}
	public void dump(PrintWriter iPrinter)
	{
		switch(type)
		{
			case IXmlModelMessage.INST_ERROR:
				iPrinter.print("+ Instanciation Error: ");
				break;
			case IXmlModelMessage.INST_WARNING:
				iPrinter.print("+ Instanciation Warning: ");
				break;
			case IXmlModelMessage.INST_INFO:
				iPrinter.print("+ Instanciation Info: ");
				break;
			case IXmlModelMessage.VALID_ERROR:
				iPrinter.print("+ Validation Error: ");
				break;
			case IXmlModelMessage.VALID_WARNING:
				iPrinter.print("+ Validation Warning: ");
				break;
			case IXmlModelMessage.VALID_INFO:
				iPrinter.print("+ Validation Info: ");
				break;
		}
		iPrinter.println(msg);
		
		if(error != null){
			error.printStackTrace(iPrinter);
		}
	}
}