/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xmlmap.impl;

import java.io.PrintWriter;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;


public class ObjInfo
{
	private ObjInfo parent;
//	private FieldMapping parentMapping;
//	private int childIndex = -1;
	private Object obj;
//	private ISchemaElement schemaElt;
	private List<IXmlModelMessage> messages;
	private URL xmlFile;
	private int lineNumber = -1;
	private HashMap<String,Integer> mappingName2Line;
	
	public ObjInfo(ObjInfo iParent, Object iObj)
	{
		parent = iParent;
		obj = iObj;
	}
//	public ObjInfo(ObjInfo iParent, FieldMapping iParentMapping, ISchemaElement iSchemaElt, Object iObj)
//	{
//		parent = iParent;
//		parentMapping = iParentMapping;
//		schemaElt = iSchemaElt;
//		obj = iObj;
//	}
//	public ObjInfo(ObjInfo iParent, FieldMapping iParentMapping, int iChildIdx, ISchemaElement iSchemaElt, Object iObj)
//	{
//		parent = iParent;
//		parentMapping = iParentMapping;
//		schemaElt = iSchemaElt;
//		childIndex = iChildIdx;
//		obj = iObj;
//	}
	public void setMappingLocation(String name, int line)
	{
		if(mappingName2Line == null){
			mappingName2Line = new HashMap<String,Integer>(1);
		}
		
		mappingName2Line.put(name, Integer.valueOf(line));
	}
	public int getMappingLocation(String name)
	{
		if(name == null || mappingName2Line == null){
			return lineNumber;
		}
		
		Integer mappingLine = (Integer)mappingName2Line.get(name);
		return mappingLine == null ? lineNumber : mappingLine.intValue();
	}
	public void setLocation(URL xmlFile, int lineNumber)
    {
    	this.xmlFile = xmlFile;
		this.lineNumber = lineNumber;
    }
	public URL getXmlFile()
    {
    	return xmlFile;
    }
	public int getLineNumber()
	{
		return lineNumber;
	}
	public Object getObject()
	{
		return obj;
	}
//	public ISchemaElement getSchemaElement()
//	{
//		return schemaElt;
//	}
	public ObjInfo getParent()
	{
		return parent;
	}
//	public FieldMapping getParentMapping()
//	{
//		return parentMapping;
//	}
//	public int getParentChildIndex()
//	{
//		return childIndex;
//	}
	// ====================================================================
	// === Messages management
	// ====================================================================
	public void clearMessages()
	{
		if(messages == null){
			return;
		}
		
		messages.clear();
	}
	public void addMessage(String iMappingName, int iType, String iMessage)
	{
		addMessage(iMappingName, iType, iMessage, null);
	}
	public void addMessage(String iMappingName, int iType, String iMessage, Throwable error)
	{
		if(messages == null){
			messages = new ArrayList<IXmlModelMessage>(1);
		}
		
		messages.add(new ModelMessageImpl(this, iMappingName, iType, iMessage, error));
	}
	public int countMessages(int treshold)
	{
		if(messages == null){
			return 0;
		}
		
		int count = 0;
		
		for(int i=0; i<messages.size(); i++){
			if(((ModelMessageImpl)messages.get(i)).getSeverity() <= treshold){
				count++;
			}
		}
		return count;
	}
	public List<IXmlModelMessage> getMessages(int treshold)
	{
		if(messages == null){
			return null;
		}
		
		List<IXmlModelMessage> ret = new ArrayList<IXmlModelMessage>();
		for(int i=0; i<messages.size(); i++){
			if(((ModelMessageImpl)messages.get(i)).getSeverity() <= treshold){
				ret.add(messages.get(i));
			}
		}
		return ret;
	}
	public boolean hasMessages(int treshold)
	{
		if(messages == null){
			return false;
		}
		for(int i=0; i<messages.size(); i++){
			if(((ModelMessageImpl)messages.get(i)).getSeverity() <= treshold){
				return true;
			}
		}
		return false;
	}
	public void dumpMessages(int treshold, PrintWriter iPrinter)
	{
		List<IXmlModelMessage> errors = getMessages(treshold);
		
		if(errors == null || errors.size() == 0){
			return;
		}
		
		iPrinter.println("["+getNicePath()+"]:");
		
		for(int i=0; i<errors.size(); i++){
				((ModelMessageImpl)errors.get(i)).dump(iPrinter);
		}
		
		iPrinter.println(" ");
	}
	/*
	private String getNiceName()
	{
		if(obj == null)
			return "Doc";
		
		StringBuffer sb = new StringBuffer();
		sb.append(schemaElt.getSchema().getName());
		sb.append(":");
		sb.append(schemaElt.getTagName());
		
		ElementMapping mappings = (ElementMapping)schemaElt.getSchema().getMethod().getElementMappings(obj.getClass(), true);
		FieldMapping[] v = mappings.getMappings(FieldMapping.ATTRIBUTE);
		for(int i=0; i<v.length; i++)
		{
			FieldMapping field = v[i];
			if(!field.isUseRequired())
				continue;
            try
            {
				field.getField().setAccessible(true);
            	Object value = field.getField().get(obj);
				if(value != null)
				{
					sb.append(" ");
					sb.append(field.getName());
					sb.append("=");
					sb.append("'");
					if(schemaElt.getSchema().getMethod().isEnumeration(field.getBaseClass()))
					{
						sb.append(schemaElt.getSchema().getMethod().getEnumItemName(value));
					}
					else
					{
						sb.append(String.valueOf(value));
					}
					sb.append("'");
				}
            }
            catch(IllegalArgumentException e)
            {
	            // TODO Auto-generated catch block
	            e.printStackTrace();
            }
            catch(IllegalAccessException e)
            {
	            // TODO Auto-generated catch block
	            e.printStackTrace();
            }
		}
		return sb.toString();
	}
	*/
	private String getNicePath()
	{
		if(obj == null)
		{
			if(xmlFile != null){
				return xmlFile+"(l "+(lineNumber >= 0 ? String.valueOf(lineNumber) : "?")+") > Document";
			}
			
			return "Document";
		}
		else
		{
			if(xmlFile != null){
				return xmlFile+"(l "+(lineNumber >= 0 ? String.valueOf(lineNumber) : "?")+") > "+obj.getClass().getName();
			}
			
			return obj.getClass().getName();
		}
	}
}