package org.ow2.opensuit.xmlmap.interfaces;

public interface IContentAssist {
	Proposal[] getProposals(String text, int position);

	public static class Proposal implements Comparable<Proposal>
	{
		private int type = 0;
		private String displayString;
		private String replacementString;
		private int replacementOffset;
		private int replacementLength;
		private int cursorPosition;
//		private org.eclipse.swt.graphics.Image fImage;
//		private org.eclipse.jface.text.contentassist.IContextInformation fContextInformation;
		private String additionalnfo;
		
		/**
		 * Constructor for a full replacement.
		 * @param fullProposal
		 * @param additionalnfo
		 */
		public Proposal(int type, String fullProposal, String additionalnfo) {
			this.type = type;
			this.displayString = fullProposal;
			this.replacementString = fullProposal;
			this.replacementOffset = 0;
			this.replacementLength = Integer.MAX_VALUE;
			this.cursorPosition = fullProposal.length();
			this.additionalnfo = additionalnfo;
		}
		/**
		 * Constructor for a local replacement.
		 * @param displayString the string to be displayed for the proposal
		 * @param replacementString the actual string to be inserted into the document
		 * @param replacementOffset the offset of the text to be replaced
		 * @param replacementLength the length of the text to be replaced
		 * @param cursorPosition the position of the cursor following the insert relative to replacementOffset
		 * @param additionalnfo the additional information associated with this proposal
		 */
		public Proposal(int type, String displayString, String replacementString,
				int replacementOffset, int replacementLength,
				int cursorPosition, String additionalnfo) {
			this.type = type;
			this.displayString = displayString;
			this.replacementString = replacementString;
			this.replacementOffset = replacementOffset;
			this.replacementLength = replacementLength;
			this.cursorPosition = cursorPosition;
			this.additionalnfo = additionalnfo;
		}
		public String getDisplayString() {
			return displayString;
		}
		public String getReplacementString() {
			return replacementString;
		}
		public int getReplacementOffset() {
			return replacementOffset;
		}
		public int getReplacementLength() {
			return replacementLength;
		}
		public int getCursorPosition() {
			return cursorPosition;
		}
		public String getAdditionalnfo() {
			return additionalnfo;
		}
		public int compareTo(Proposal o) {
			if(o.type != this.type)
				return this.type - o.type;
			return this.displayString.compareTo(o.displayString);
		}
		
	}
}
