/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xmlmap.interfaces;

import java.net.URL;

import org.ow2.opensuit.xmlmap.impl.IXmlModelMessage;

/**
 * This interface allows logging validation errors,
 * and trigger dependent objects initialization
 * 
 * @author psmeyers
 */
public interface IInitializationSupport
{
	/**
	 * Message Severity: information
	 */
	public static int INFO = IXmlModelMessage.VALID_INFO;
	/**
	 * Message Severity: warning
	 */
	public static int WARNING = IXmlModelMessage.VALID_WARNING;
	/**
	 * Message Severity: error (blocking)
	 */
	public static int ERROR = IXmlModelMessage.VALID_ERROR;
	
	/**
	 * Logs a validity message related to the given IXmlObject object.
	 * @param iFromNode The checking IXmlObject
	 * @param iMappingName The possible field mapping name related to this message
	 * @param iSeverity The message severity (INFO / WARNING / ERROR)
	 * @param iMessage The message text
	 */
	void addValidationMessage(Object iFromNode, String iMappingName, int iSeverity, String iMessage);
	void addValidationMessage(Object iFromNode, String iMappingName, int iSeverity, String iMessage, Throwable error);
	
	/**
	 * This method may be used during the initialization process.
	 * Calling this method triggers the child object initialization, then returns
	 * whether the required object have been properly initialized.
	 * Warning: if calling this method with a parent element object, this will throw a "cyclic dependencies" runtime error.
	 * @param iChild object corresponding to a child XML element.
	 * @return <code>true</code> if the object was properly initialized, <code>false</code> otherwise (if any error occurred).
	 */
	boolean initialize(Object iChild);
	
	/**
	 * This method may be used during the initialization process.
	 * Calling this method loads a dependent object from XML and adds objects to the
	 * current model.
	 * Validation will be triggered afterwards.
	 */
	Object instantiate(URL xmlFileUrl) throws Exception;
}
