package org.ow2.opensuit.xmlmap.tests;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Vector;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;


/**
 * The base class for instantiating an object model that conform to XML mapping rules.
 */
public class TestSAX
{
	public static void main(String[] args) throws MalformedURLException, SAXException, IOException, ParserConfigurationException
	{
//		File f = new File(args[0]);
		File f = new File("C:/Dev/EclipseProj/samples/movies_db/src/main/webapp/WEB-INF/MoviesDBApp.xml");
//		File f = new File("E:/temp/MoviesDBApp_ko.xml");
		instantiate(f.toURL());
	}
	
	private URL mainDoc;
	
	private TestSAX(URL iMainDocUrl)
	{
		mainDoc = iMainDocUrl;
	}


	public final static TestSAX instantiate(URL iXmlFileUrl) throws SAXException, IOException, ParserConfigurationException
	{
		TestSAX doc = new TestSAX(iXmlFileUrl);
		doc.instantiateMainDoc();
		return doc;
	}
	
	private void instantiateMainDoc() throws SAXException, IOException, ParserConfigurationException
	{
		// --- 1: parse main document
		InputStream in = mainDoc.openStream();
		SAXParserFactory parserFactory = SAXParserFactory.newInstance();
		parserFactory.setNamespaceAware(true);
//		parserFactory.setValidating(true);
//		parserFactory.setFeature(name, value)
		SAXParser parser = parserFactory.newSAXParser();
//		parser.setProperty(name, value)Property(name, value)
		
		// --- now parse
		//TODO: wrap with an input stream that is aware of line and column
		parser.parse(in, new SAXHandler());
		in.close();
		
		/*
//        XMLReader saxReader = XMLReaderFactory.createXMLReader("org.apache.xerces.parsers.SAXParser");
		InputSource source = new InputSource(in);
        XMLReader saxReader = XMLReaderFactory.createXMLReader("org.apache.crimson.parser.XMLReaderImpl");
//        XMLReader saxReader = XMLReaderFactory.createXMLReader();
        saxReader.setContentHandler(new SAXHandler());
        saxReader.parse(source);
        */
	}

	// ======================================================================
	// === SAX Handler
	// ======================================================================
	private class SAXHandler extends DefaultHandler
	{
		private Locator locator = null;
		private Vector infoAncestors = new Vector();
		
		public void setDocumentLocator(Locator locator)
		{
			this.locator = locator;
		}
		private void trace(String message)
		{
			if(locator == null){
				System.out.println("[?] "+message);
			}else{
				System.out.println("[l"+locator.getLineNumber()+" c"+locator.getColumnNumber()+"] "+message);
			}
		}
		public void startDocument() throws SAXException
		{
			trace("startDocument()");
		}
		public void endDocument() throws SAXException
		{
			trace("endDocument()");
		}
		public void startElement(String uri, String localName, String name, Attributes attribs) throws SAXException
		{
			trace("startElement('"+uri+"', '"+localName+"', '"+name+"')");
		}
		public void characters(char[] chars, int start, int length) throws SAXException
		{
//			trace("characters: ["+new String(chars, start, length)+"]");
//			trace("characters: "+length);
		}
		public void endElement(String uri, String localName, String name) throws SAXException
		{
			trace("endElement('"+uri+"', '"+localName+"', '"+name+"')");
		}
		
		public void startPrefixMapping(String prefix, String uri) throws SAXException
		{
			trace("startPrefixMapping('"+prefix+"', '"+uri+"')");
		}
		public void endPrefixMapping(String prefix) throws SAXException
		{
			trace("endPrefixMapping('"+prefix+"')");
		}
		public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException
		{
			trace("ignorableWhitespace(): "+length);
		}
		public void processingInstruction(String target, String data) throws SAXException
		{
			trace("processingInstruction('"+target+"', '"+data+"')");
		}
		public void skippedEntity(String name) throws SAXException
		{
			trace("skippedEntity('"+name+"')");
		}
		/*public InputSource resolveEntity(String publicId, String systemId) throws SAXException
		{
			trace("resolveEntity('"+publicId+"', '"+systemId+"')");
			return super.resolveEntity(publicId, systemId);
		}*/
		public void notationDecl(String name, String publicId, String systemId) throws SAXException
		{
			trace("notationDecl('"+name+"', '"+publicId+"', '"+systemId+"')");
			super.notationDecl(name, publicId, systemId);
		}
		public void unparsedEntityDecl(String name, String publicId, String systemId, String notationName) throws SAXException
		{
			trace("unparsedEntityDecl('"+name+"', '"+publicId+"', '"+systemId+"')");
			super.unparsedEntityDecl(name, publicId, systemId, notationName);
		}
		public void error(SAXParseException e) throws SAXException
		{
			trace("error('"+e.getMessage()+"')");
			super.error(e);
		}
		public void fatalError(SAXParseException e) throws SAXException
		{
			trace("fatalError('"+e.getMessage()+"')");
			super.fatalError(e);
		}
		public void warning(SAXParseException e) throws SAXException
		{
			trace("warning('"+e.getMessage()+"')");
			super.warning(e);
		}
		
	}

}
