/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xmlmap.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.ext.LexicalHandler;
import org.xml.sax.helpers.DefaultHandler;

public class XML2HTML
{

	/**
	 * @param args
	 * @throws IOException 
	 * @throws SAXException 
	 * @throws ParserConfigurationException 
	 * @throws ParserConfigurationException 
	 * @throws IOException 
	 * @throws SAXException 
	 * @throws IOException 
	 * @throws ParserConfigurationException 
	 * @throws IOException 
	 * @throws SAXException 
	 */
	public static void main(String[] args) throws ParserConfigurationException, SAXException, IOException
	{
		if(args.length < 1)
		{
			System.out.println("Usage: XML2HTML <input xml file or dir> [output file or dir]");
			return;
		}
		// --- 1: retrieve xml file
		File inXmlFile = new File(args[0]);
		if(!inXmlFile.exists())
		{
			System.out.println("File ["+args[0]+"] not found.");
	        return;
		}
		File inXmlDir = inXmlFile.isDirectory() ? inXmlFile : inXmlFile.getParentFile();

		// --- 2: retrieve output html (if any)
		File outHtmlFile = null;
		if(args.length > 1)
		{
			outHtmlFile = new File(args[1]);
		}

		if(inXmlFile.isFile())
		{
			// --- single file format
			if(outHtmlFile == null)
			{
				outHtmlFile = new File(inXmlFile.getParentFile(), inXmlFile.getName().replace('.', '_')+".html");
			}
			else if(outHtmlFile.exists() && outHtmlFile.isDirectory())
			{
				outHtmlFile = new File(outHtmlFile, inXmlFile.getName().replace('.', '_')+".html");
			}
			// --- export and format
			new XML2HTML().format( inXmlFile, outHtmlFile);
		}
		else
		{
			// --- directory processing
			File outHtmlDir = inXmlDir;
			if(outHtmlFile != null)
			{
				if(!outHtmlFile.exists()){
					// create directory
					outHtmlFile.mkdirs();
				}
				
				if(outHtmlFile.exists() && outHtmlFile.isDirectory()){
					outHtmlDir = outHtmlFile;
				}
			}
			new XML2HTML().recurFormat(inXmlDir, outHtmlDir);
		}
	}
	
	private PrintWriter writer;
//	private int indent = 0;
	
	private void recurFormat(File inXmlDir, File outHtmlDir)
	{
		File[] files = inXmlDir.listFiles();
		if(files != null)
		{
			for(int i=0; i<files.length; i++)
			{
				File f = files[i];
				if(f.isDirectory())
				{
					// --- process directory
					recurFormat(f, new File(outHtmlDir, f.getName()));
				}
				else if(f.getName().endsWith(".xml"))
				{
					// --- format file
					File outHtmlFile = new File(outHtmlDir, f.getName().replace('.', '_')+".html");
					try
					{
						format(f, outHtmlFile);
					}
					catch (Exception e)
					{
						System.err.println("Error occured while formatting file "+f);
						e.printStackTrace();
					}
				}
			}
		}
	}
	private void format(File inXmlFile, File outHtmlFile) throws FileNotFoundException, ParserConfigurationException, SAXException, IOException
	{
		outHtmlFile.getParentFile().mkdirs();
		format(new FileInputStream(inXmlFile), new FileOutputStream(outHtmlFile));
	}
	
	public void format(InputStream input, OutputStream output) throws ParserConfigurationException, SAXException, IOException
	{
		// --- 2: parse main document
		SAXParserFactory parserFactory = SAXParserFactory.newInstance();
		parserFactory.setNamespaceAware(true);
//		parserFactory.setValidating(true);
//		parserFactory.setFeature(name, value)
		SAXParser parser = parserFactory.newSAXParser();
//		parser.setProperty(name, value)Property(name, value)

		writer = new PrintWriter(output);
		SAXHandler handler = new SAXHandler(writer);
		parser.setProperty("http://xml.org/sax/properties/lexical-handler", handler);

		// --- now parse and format
		openHtml();
		parser.parse(input, handler);
//		writeElement(doc.getDocumentElement());
		closeHtml();
		
		writer.flush();
		writer.close();
		input.close();
	}
	private void openHtml()
	{
		writer.println("<html>");
		writer.println("<head>");
		//TODO: title, generated by, date, ...
		writer.println("</head>");
		
		writer.println("<style type='text/css'>");
		writer.println("pre.XML {");
		writer.println("	font-family: courier;");
		writer.println("	font-size: 9pt;");
		writer.println("	background-color: #FFFFFF;");
		writer.println("}");
		writer.println("span.Element {");
		writer.println("	color: #3F7F7F;");
		writer.println("}");
		/*
		writer.println("span.Hover {");
		writer.println("	background-color: #CCCCCC;");
		writer.println("}");
		*/
		/*
		writer.println("span.Element:hover {");
		writer.println("	background-color: #CCCCCC;");
		writer.println("}");
		writer.println("span.Element:hover span.Element {");
		writer.println("	background-color: none;");
		writer.println("}");
		*/
		writer.println("span.AttrName {");
		writer.println("	color: #7F007F;");
		writer.println("}");
		writer.println("span.AttrValue {");
		writer.println("	color: #2A00FF;");
		writer.println("}");
		writer.println("span.Comment {");
		writer.println("	color: #3F5FBF;");
		writer.println("}");
		writer.println("span.Text {");
		writer.println("	color: #000000;");
		writer.println("}");
		writer.println("</style>");

		/*
		writer.println("<script>");
		writer.println("var HOVER_EL = null;");
		writer.println("function onMouseOver(e)");
		writer.println("{");
		writer.println("	var o = window.event ? event.srcElement : e.target;");
		writer.println("	if(e.nodeName != 'SPAN' || e.className != 'Element')");
		writer.println("		return;");
		writer.println("	if(HOVER_EL != null)");
		writer.println("		HOVER_EL.className = 'Element';");
		writer.println("	HOVER_EL = o;");
		writer.println("	HOVER_EL.className = 'Element Hover';");
		writer.println("}");
		writer.println("</script>");
		*/

//		writer.println("<body onmouseover='onMouseOver(event)'>");
		writer.println("<body>");
		writer.println("<pre class='XML'>");
	}
	private void closeHtml()
	{
		writer.println("</pre>");
		writer.println("</body>");
		writer.println("</html>");
	}
	/*
	private void writeIndentation()
	{
		for(int i=0; i<indent; i++)
		{
			writer.print("&nbsp;&nbsp;");
		}
	}
	*/
	/*
	private boolean isEmpty(CharacterData node)
	{
		return node.getData() == null || node.getData().trim().length() == 0;
	}
	private boolean isMultiLine(CharacterData node)
	{
		return node.getData() != null && node.getData().trim().indexOf('\n') > 0;
	}
	*/
	/*
	private void writeCData(CDATASection node)
	{
//		if(node.getData() == null || node.getData().trim().length() == 0)
//			return;
//		writeIndentation();
		writer.print("&lt;![CDATA[<span class=Text>");
		writer.print(encode2HTML(node.getData(), false));
		writer.print("</span>]]&gt;");
	}
	private void writeComment(Comment node)
	{
//		writeIndentation();
		writer.print("<span class=Comment>&lt;!-- ");
		writer.print(encode2HTML(node.getData(), false));
		writer.print("--&gt;</span>");
	}
	private void writeText(Text node)
	{
//		if(node.getData() == null || node.getData().trim().length() == 0)
//			return;
//		writeIndentation();
		writer.print("<span class=Text>");
		writer.print(encode2HTML(node.getData(), false));
		writer.print("</span>");
	}
	private void writeElement(Element node)
	{
		// --- 1: open tag
//		writeIndentation();
		writer.print("&lt;");
		writer.print(node.getNodeName());
		
		// --- 2: write attributes
		NamedNodeMap attributes = node.getAttributes();
		if(attributes != null)
		{
			for(int i=0; i<attributes.getLength(); i++)
			{
				Attr attr = (Attr)attributes.item(i);
				writer.print(" ");
				writer.print("<span class=AttrName>");
				writer.print(attr.getName());
				writer.print("</span>");
				writer.print("=");
				writer.print("<span class=AttrValue>");
				writer.print("\"");
				writer.print(attr.getValue());
				writer.print("\"");
				writer.print("</span>");
			}
		}
		
		// --- 3: write children nodes
		if(!node.hasChildNodes())
		{
			// --- inline node
			writer.print("/&gt;");
			return;
		}
		
		// --- close opening tag
		writer.print("&gt;");
		
		// --- can we write children inline ?
		NodeList children = node.getChildNodes();
		boolean writeChildrenInline = true;
		int countChildren = 0;
		for(int i=0; writeChildrenInline && i<children.getLength(); i++)
		{
			Node child = children.item(i);
			switch(child.getNodeType())
			{
				case Node.CDATA_SECTION_NODE:
					if(isEmpty((CharacterData)child))
						break;
					if(isMultiLine((CharacterData)child))
						writeChildrenInline = false;
					countChildren++;
					break;
				case Node.TEXT_NODE:
					if(isEmpty((CharacterData)child))
						break;
					if(isMultiLine((CharacterData)child))
						writeChildrenInline = false;
					countChildren++;
					break;
				case Node.ELEMENT_NODE:
					writeChildrenInline = false;
//					countChildren++;
					break;
				case Node.COMMENT_NODE:
					writeChildrenInline = false;
					break;
			}
			if(countChildren > 1)
				writeChildrenInline = false;
		}
		
		// --- write children
		if(!writeChildrenInline)
			writer.println();
		
		indent++;
		for(int i=0; i<children.getLength(); i++)
		{
			Node child = children.item(i);
			switch(child.getNodeType())
			{
				case Node.CDATA_SECTION_NODE:
					if(isEmpty((CharacterData)child))
						continue;
					if(!writeChildrenInline)
						writeIndentation();
					writeCData((CDATASection)child);
					if(!writeChildrenInline)
						writer.println();
					break;
				case Node.TEXT_NODE:
					if(isEmpty((CharacterData)child))
						continue;
					if(!writeChildrenInline)
						writeIndentation();
					writeText((Text)child);
					if(!writeChildrenInline)
						writer.println();
					break;
				case Node.ELEMENT_NODE:
					if(!writeChildrenInline)
						writeIndentation();
					writeElement((Element)child);
					if(!writeChildrenInline)
						writer.println();
					break;
				case Node.COMMENT_NODE:
					if(!writeChildrenInline)
						writeIndentation();
					writeComment((Comment)child);
					if(!writeChildrenInline)
						writer.println();
					break;
			}
		}
		indent--;
		
		// --- write closing tag
		if(!writeChildrenInline)
			writeIndentation();
		writer.print("&lt;/");
		writer.print(node.getNodeName());
		writer.print("&gt;");
	}
	*/
	
	public static String encode2HTML(char[] text, int start, int len, boolean iNBSP)
	{
		int nbChars = len;
		StringBuffer sb = new StringBuffer(nbChars);
		for(int i=0; i<len; i++)
		{
			char c = text[start+i];
			if(iNBSP && c == ' ')
			{
				sb.append("&nbsp;");
			}
			else
			{
				switch(c)
				{
				case '\n': // \n turned into \n (as in PRE block)
//					sb.append("<br>");
					sb.append("\n");
					break;
				case '\r': // \r ignored
//					sb.append(" ");
					break;
				case '\t': // TAB is turned into 4 spaces
//					sb.append("&nbsp;&nbsp;&nbsp;&nbsp;");
					sb.append("    ");
					break;
				case '<':
					sb.append("&lt;");
					break;
				case '>':
					sb.append("&gt;");
					break;
				case '&':
					sb.append("&amp;");
					break;
				case '\"':
					sb.append("&quot;");
					break;
				case '\'':
					sb.append("&#39;");
					break;
				default:
					sb.append(c);
					break;
				}
			}
		}
		return sb.toString();
	}
	// ======================================================================
	// === SAX Handler
	// ======================================================================
	private class SAXHandler extends DefaultHandler implements LexicalHandler
	{
		private Element openedElt;
		private PrintWriter htmlOutput;
		private Locator locator = null;
		
		public SAXHandler(PrintWriter htmlOutput)
		{
			this.htmlOutput = htmlOutput;
		}
		public void setDocumentLocator(Locator locator)
		{
			this.locator = locator;
		}
		private void trace(String message)
		{
			/*
			if(locator == null)
				System.out.println("[?] "+message);
			else
				System.out.println("[l"+locator.getLineNumber()+" c"+locator.getColumnNumber()+"] "+message);
			*/
		}
		public void startDocument() throws SAXException
		{
			trace("startDocument()");
		}
		public void endDocument() throws SAXException
		{
			trace("endDocument()");
		}
		private boolean flushElement(boolean inline)
		{
			if(openedElt == null){
				return false;
			}
			
			// --- 1: open tag
//			writeIndentation();
			writer.print("<span class='Element'>");
			writer.print("&lt;");
			writer.print(openedElt.name);
			
			// --- 2: write attributes
			if(openedElt.attribs != null)
			{
				for(int i=0; i<openedElt.attribs.getLength(); i++)
				{
//					Attr attr = (Attr)attribs.item(i);
					writer.print(" ");
					writer.print("<span class='AttrName'>");
					writer.print(openedElt.attribs.getLocalName(i));
					writer.print("</span>");
					writer.print("=");
					writer.print("<span class='AttrValue'>");
					writer.print("\"");
					writer.print(openedElt.attribs.getValue(i));
					writer.print("\"");
					writer.print("</span>");
				}
			}
			
			// --- 3: write children nodes
			if(inline)
			{
				// --- inline node
				writer.print("/&gt;");
				writer.print("</span>");
			} else{
				writer.print("&gt;");
			}
			
			openedElt = null;
			
			return true;
		}
		public void startElement(String uri, String localName, String name, Attributes attribs) throws SAXException
		{
			trace("startElement('"+uri+"', '"+localName+"', '"+name+"')");
			
			flushElement(false);
			
			openedElt = new Element(uri, localName, name, attribs);
		}
		public void characters(char[] chars, int start, int length) throws SAXException
		{
			trace("characters: \""+new String(chars, start, length)+"\" ("+length+")");
			flushElement(false);

//			trace("characters: "+length);
			boolean hasText = !isWhitespaces(chars, start, length);
			
			if(hasText){
				htmlOutput.write("<span class='Text'>");
			}
			
			htmlOutput.write(encode2HTML(chars, start, length, false));
			
			if(hasText){
				htmlOutput.write("</span>");
			}
			
//			htmlOutput.write(chars, start, length);
		}
		private boolean isWhitespaces(char[] chars, int start, int length)
		{
			for(int i=0; i<length; i++)
			{
				if(!Character.isWhitespace(chars[i+start])){
					return false;
				}
			}
			return true;
		}
		public void endElement(String uri, String localName, String name) throws SAXException
		{
			trace("endElement('"+uri+"', '"+localName+"', '"+name+"')");
			if(!flushElement(true))
			{
				// --- write closing tag
	//			if(!writeChildrenInline)
	//				writeIndentation();
				writer.print("&lt;/");
				writer.print(name);
				writer.print("&gt;");
				writer.print("</span>");
			}
		}
		
		public void startPrefixMapping(String prefix, String uri) throws SAXException
		{
			trace("startPrefixMapping('"+prefix+"', '"+uri+"')");
		}
		public void endPrefixMapping(String prefix) throws SAXException
		{
			trace("endPrefixMapping('"+prefix+"')");
		}
		public void ignorableWhitespace(char[] chars, int start, int length) throws SAXException
		{
			trace("ignorableWhitespace(): "+length);
			flushElement(false);
			
			htmlOutput.write(encode2HTML(chars, start, length, false));
//			htmlOutput.write(chars, start, length);
		}
		public void processingInstruction(String target, String data) throws SAXException
		{
			trace("processingInstruction('"+target+"', '"+data+"')");
		}
		public void skippedEntity(String name) throws SAXException
		{
			trace("skippedEntity('"+name+"')");
		}
		/*public InputSource resolveEntity(String publicId, String systemId) throws SAXException
		{
			trace("resolveEntity('"+publicId+"', '"+systemId+"')");
			return super.resolveEntity(publicId, systemId);
		}*/
		public void notationDecl(String name, String publicId, String systemId) throws SAXException
		{
			trace("notationDecl('"+name+"', '"+publicId+"', '"+systemId+"')");
			super.notationDecl(name, publicId, systemId);
		}
		public void unparsedEntityDecl(String name, String publicId, String systemId, String notationName) throws SAXException
		{
			trace("unparsedEntityDecl('"+name+"', '"+publicId+"', '"+systemId+"')");
			super.unparsedEntityDecl(name, publicId, systemId, notationName);
		}
		public void error(SAXParseException e) throws SAXException
		{
			trace("error('"+e.getMessage()+"')");
			super.error(e);
		}
		public void fatalError(SAXParseException e) throws SAXException
		{
			trace("fatalError('"+e.getMessage()+"')");
			super.fatalError(e);
		}
		public void warning(SAXParseException e) throws SAXException
		{
			trace("warning('"+e.getMessage()+"')");
			super.warning(e);
		}
		// ===== Lexical Handler
		public void comment(char[] chars, int start, int length) throws SAXException
		{
			trace("comment: ["+new String(chars, start, length)+"]");
			flushElement(false);
			
			writer.print("<span class='Comment'>&lt;!--");
			writer.print(encode2HTML(chars, start, length, false));
			writer.print("--&gt;</span>");
		}
		public void startCDATA() throws SAXException
		{
			trace("startCDATA");
			flushElement(false);
			writer.print("&lt;![CDATA[");
		}
		public void endCDATA() throws SAXException
		{
			trace("endCDATA");
			flushElement(false);
			writer.print("]]&gt;");
		}
		public void startDTD(String name, String publicId, String systemId) throws SAXException
		{
			trace("startDTD");
		}
		public void endDTD() throws SAXException
		{
			trace("endDTD");
		}
		public void startEntity(String name) throws SAXException
		{
			trace("startEntity: "+name);
		}
		public void endEntity(String name) throws SAXException
		{
			trace("endEntity: "+name);
		}
	}
	private static class Element
	{
		String uri;
		String localName;
		String name;
		Attributes attribs;
		
		public Element(String uri, String localName, String name, Attributes attribs)
		{
			this.uri = uri;
			this.localName = localName;
			this.name = name;
			this.attribs = attribs;
		}
	}
}
